@@ -1,6 +1,7 @@
 package common
 
 import (
+	"bytes"
 	"context"
 	"errors"
 	"fmt"
@@ -55,6 +56,115 @@ func TestBuildRun(t *testing.T) {
 	runSuccessfulMockBuild(t, func(options ExecutorPrepareOptions) error { return nil })
 }
 
+func TestBuildPanic(t *testing.T) {
+	panicFn := func(mock.Arguments) {
+		panic("panic message")
+	}
+
+	tests := map[string]struct {
+		setupMockExecutor func(*MockExecutor)
+	}{
+		"prepare": {
+			setupMockExecutor: func(executor *MockExecutor) {
+				executor.On("Prepare", mock.Anything, mock.Anything, mock.Anything).
+					Run(panicFn).Once()
+			},
+		},
+		"run": {
+			setupMockExecutor: func(executor *MockExecutor) {
+				executor.On("Prepare", mock.Anything, mock.Anything, mock.Anything).
+					Return(nil).Once()
+				executor.On("Finish", mock.Anything).Once()
+				executor.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"})
+				executor.On("Run", mock.Anything).Run(panicFn).Once()
+				executor.On("Cleanup").Once()
+			},
+		},
+		"cleanup": {
+			setupMockExecutor: func(executor *MockExecutor) {
+				executor.On("Prepare", mock.Anything, mock.Anything, mock.Anything).
+					Return(nil).Once()
+				executor.On("Finish", mock.Anything).Once()
+				executor.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"})
+				executor.On("Run", mock.Anything).Once()
+				executor.On("Cleanup").Run(panicFn).Once()
+			},
+		},
+		"shell": {
+			setupMockExecutor: func(executor *MockExecutor) {
+				executor.On("Prepare", mock.Anything, mock.Anything, mock.Anything).
+					Return(nil).Once()
+				executor.On("Finish", mock.Anything).Once()
+				executor.On("Shell").Run(panicFn)
+				executor.On("Cleanup").Once()
+			},
+		},
+		"run+cleanup": {
+			setupMockExecutor: func(executor *MockExecutor) {
+				executor.On("Prepare", mock.Anything, mock.Anything, mock.Anything).
+					Return(nil).Once()
+				executor.On("Finish", mock.Anything).Once()
+				executor.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"})
+				executor.On("Run", mock.Anything).Run(panicFn).Once()
+				executor.On("Cleanup").Run(panicFn).Once()
+			},
+		},
+		"finish": {
+			setupMockExecutor: func(executor *MockExecutor) {
+				executor.On("Prepare", mock.Anything, mock.Anything, mock.Anything).
+					Return(nil).Once()
+				executor.On("Finish", mock.Anything).Run(panicFn).Once()
+				executor.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"})
+				executor.On("Run", mock.Anything).Once()
+				executor.On("Cleanup").Once()
+			},
+		},
+		"finish+cleanup+shell": {
+			setupMockExecutor: func(executor *MockExecutor) {
+				executor.On("Prepare", mock.Anything, mock.Anything, mock.Anything).
+					Return(nil).Once()
+				executor.On("Finish", mock.Anything).Run(panicFn).Once()
+				executor.On("Shell").Run(panicFn).Return(&ShellScriptInfo{Shell: "script-shell"})
+				executor.On("Cleanup").Run(panicFn).Once()
+			},
+		},
+		"run+finish+cleanup": {
+			setupMockExecutor: func(executor *MockExecutor) {
+				executor.On("Prepare", mock.Anything, mock.Anything, mock.Anything).
+					Return(nil).Once()
+				executor.On("Finish", mock.Anything).Run(panicFn).Once()
+				executor.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"})
+				executor.On("Run", mock.Anything).Run(panicFn).Once()
+				executor.On("Cleanup").Run(panicFn).Once()
+			},
+		},
+	}
+
+	for tn, tt := range tests {
+		t.Run(tn, func(t *testing.T) {
+			executor, provider := setupMockExecutorAndProvider()
+			defer executor.AssertExpectations(t)
+			defer provider.AssertExpectations(t)
+
+			tt.setupMockExecutor(executor)
+
+			RegisterExecutorProvider(t.Name(), provider)
+
+			res, err := GetSuccessfulBuild()
+			require.NoError(t, err)
+
+			cfg := &RunnerConfig{}
+			cfg.Executor = t.Name()
+			build, err := NewBuild(res, cfg, nil, nil)
+			require.NoError(t, err)
+			var out bytes.Buffer
+			err = build.Run(&Config{}, &Trace{Writer: &out})
+			assert.EqualError(t, err, "panic: panic message")
+			assert.Contains(t, out.String(), "panic: panic message")
+		})
+	}
+}
+
 func TestJobImageExposed(t *testing.T) {
 	tests := map[string]struct {
 		image           string
@@ -885,9 +995,11 @@ func TestSkipBuildStageFeatureFlag(t *testing.T) {
 		"false",
 	}
 
-	s := MockShell{}
+	s := new(MockShell)
+	defer s.AssertExpectations(t)
+
 	s.On("GetName").Return("skip-build-stage-shell")
-	RegisterShell(&s)
+	RegisterShell(s)
 
 	for _, value := range featureFlagValues {
 		t.Run(value, func(t *testing.T) {
@@ -1265,11 +1377,11 @@ func TestBuildFinishTimeout(t *testing.T) {
 
 func TestProjectUniqueName(t *testing.T) {
 	tests := map[string]struct {
-		build        Build
+		build        *Build
 		expectedName string
 	}{
 		"project non rfc1132 unique name": {
-			build: Build{
+			build: &Build{
 				Runner: &RunnerConfig{
 					RunnerCredentials: RunnerCredentials{
 						Token: "Ze_n8E6en622WxxSg4r8",
@@ -1285,7 +1397,7 @@ func TestProjectUniqueName(t *testing.T) {
 			expectedName: "runner-zen8e6e-project-1234567890-concurrent-0",
 		},
 		"project non rfc1132 unique name longer than 63 char": {
-			build: Build{
+			build: &Build{
 				Runner: &RunnerConfig{
 					RunnerCredentials: RunnerCredentials{
 						Token: "Ze_n8E6en622WxxSg4r8",
@@ -1301,7 +1413,7 @@ func TestProjectUniqueName(t *testing.T) {
 			expectedName: "runner-zen8e6e-project-123456789012345-concurrent-1234567890123",
 		},
 		"project normal unique name": {
-			build: Build{
+			build: &Build{
 				Runner: &RunnerConfig{
 					RunnerCredentials: RunnerCredentials{
 						Token: "xYzWabc-Ij3xlKjmoPO9",
@@ -1471,3 +1583,115 @@ func runSuccessfulMockBuild(t *testing.T, prepareFn func(options ExecutorPrepare
 
 	return build
 }
+
+func TestSecretsResolving(t *testing.T) {
+	exampleVariables := JobVariables{
+		{Key: "key", Value: "value"},
+	}
+
+	setupFailureExecutorMocks := func(t *testing.T) (*MockExecutorProvider, func()) {
+		e := new(MockExecutor)
+		p := new(MockExecutorProvider)
+
+		p.On("CanCreate").Return(true).Once()
+		p.On("GetDefaultShell").Return("bash").Once()
+		p.On("GetFeatures", mock.Anything).Return(nil).Once()
+
+		assertFn := func() {
+			e.AssertExpectations(t)
+			p.AssertExpectations(t)
+		}
+
+		return p, assertFn
+	}
+
+	secrets := Secrets{
+		"TEST_SECRET": Secret{
+			Vault: &VaultSecret{},
+		},
+	}
+
+	tests := map[string]struct {
+		secrets                 Secrets
+		resolverCreationError   error
+		prepareExecutorProvider func(t *testing.T) (*MockExecutorProvider, func())
+		returnVariables         JobVariables
+		resolvingError          error
+		expectedVariables       JobVariables
+		expectedError           error
+	}{
+		"secrets not present": {
+			prepareExecutorProvider: func(t *testing.T) (*MockExecutorProvider, func()) {
+				return setupSuccessfulMockExecutor(t, func(options ExecutorPrepareOptions) error { return nil })
+			},
+			expectedError: nil,
+		},
+		"error on creating resolver": {
+			secrets:                 secrets,
+			resolverCreationError:   assert.AnError,
+			prepareExecutorProvider: setupFailureExecutorMocks,
+			expectedError:           assert.AnError,
+		},
+		"error on secrets resolving": {
+			secrets:                 secrets,
+			prepareExecutorProvider: setupFailureExecutorMocks,
+			returnVariables:         exampleVariables,
+			resolvingError:          assert.AnError,
+			expectedVariables:       nil,
+			expectedError:           assert.AnError,
+		},
+		"secrets resolved": {
+			secrets: secrets,
+			prepareExecutorProvider: func(t *testing.T) (*MockExecutorProvider, func()) {
+				return setupSuccessfulMockExecutor(t, func(options ExecutorPrepareOptions) error { return nil })
+			},
+			returnVariables:   exampleVariables,
+			resolvingError:    nil,
+			expectedVariables: exampleVariables,
+			expectedError:     nil,
+		},
+	}
+
+	for tn, tt := range tests {
+		t.Run(tn, func(t *testing.T) {
+			secretsResolverMock := new(MockSecretsResolver)
+			defer secretsResolverMock.AssertExpectations(t)
+
+			p, assertFn := tt.prepareExecutorProvider(t)
+			defer assertFn()
+
+			RegisterExecutorProvider(t.Name(), p)
+
+			successfulBuild, err := GetSuccessfulBuild()
+			require.NoError(t, err)
+
+			successfulBuild.Secrets = tt.secrets
+
+			if tt.resolverCreationError == nil && tt.secrets != nil {
+				secretsResolverMock.On("Resolve", tt.secrets).
+					Return(tt.returnVariables, tt.resolvingError).
+					Once()
+			}
+
+			rc := new(RunnerConfig)
+			rc.RunnerSettings.Executor = t.Name()
+
+			build, err := NewBuild(successfulBuild, rc, nil, nil)
+			assert.NoError(t, err)
+
+			build.secretsResolver = func(_ logger, _ SecretResolverRegistry) (SecretsResolver, error) {
+				return secretsResolverMock, tt.resolverCreationError
+			}
+
+			err = build.Run(&Config{}, &Trace{Writer: os.Stdout})
+
+			assert.Equal(t, tt.expectedVariables, build.secretsVariables)
+
+			if tt.expectedError != nil {
+				assert.True(t, errors.As(err, &tt.expectedError))
+				return
+			}
+			assert.NoError(t, err)
+		})
+	}
+}
