@@ -24,13 +24,13 @@ import (
 	"github.com/stretchr/testify/require"
 	api "k8s.io/api/core/v1"
 	kubeerrors "k8s.io/apimachinery/pkg/api/errors"
-	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/intstr"
 	"k8s.io/client-go/kubernetes"
 	"k8s.io/client-go/rest/fake"
 
 	"gitlab.com/gitlab-org/gitlab-runner/common"
+	"gitlab.com/gitlab-org/gitlab-runner/common/buildtest"
 	"gitlab.com/gitlab-org/gitlab-runner/executors"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/container/helperimage"
@@ -43,13 +43,19 @@ import (
 
 type featureFlagTest func(t *testing.T, flagName string, flagValue bool)
 
+func mustCreateResourceList(t *testing.T, cpu, memory string) api.ResourceList {
+	resources, err := createResourceList(cpu, memory)
+	require.NoError(t, err)
+
+	return resources
+}
+
 func TestRunTestsWithFeatureFlag(t *testing.T) {
 	tests := map[string]featureFlagTest{
 		"testKubernetesSuccessRun":              testKubernetesSuccessRunFeatureFlag,
 		"testKubernetesMultistepRun":            testKubernetesMultistepRunFeatureFlag,
 		"testKubernetesTimeoutRun":              testKubernetesTimeoutRunFeatureFlag,
 		"testKubernetesBuildFail":               testKubernetesBuildFailFeatureFlag,
-		"testKubernetesBuildAbort":              testKubernetesBuildAbortFeatureFlag,
 		"testKubernetesBuildCancel":             testKubernetesBuildCancelFeatureFlag,
 		"testVolumeMounts":                      testVolumeMountsFeatureFlag,
 		"testVolumes":                           testVolumesFeatureFlag,
@@ -214,7 +220,7 @@ func testKubernetesTimeoutRunFeatureFlag(t *testing.T, featureFlagName string, f
 			},
 		},
 	}
-	build.RunnerInfo.Timeout = 10 //seconds
+	build.RunnerInfo.Timeout = 10 // seconds
 	setBuildFeatureFlag(build, featureFlagName, featureFlagValue)
 
 	err = build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
@@ -247,87 +253,30 @@ func testKubernetesBuildFailFeatureFlag(t *testing.T, featureFlagName string, fe
 
 	err = build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
 	require.Error(t, err, "error")
-	assert.IsType(t, err, &common.BuildError{})
+	assert.IsType(t, &common.BuildError{}, err)
 	assert.Contains(t, err.Error(), "command terminated with exit code 1")
 }
 
-func testKubernetesBuildAbortFeatureFlag(t *testing.T, featureFlagName string, featureFlagValue bool) {
-	if helpers.SkipIntegrationTests(t, "kubectl", "cluster-info") {
-		return
-	}
-
-	failedBuild, err := common.GetRemoteFailedBuild()
-	assert.NoError(t, err)
-	build := &common.Build{
-		JobResponse: failedBuild,
-		Runner: &common.RunnerConfig{
-			RunnerSettings: common.RunnerSettings{
-				Executor: "kubernetes",
-				Kubernetes: &common.KubernetesConfig{
-					PullPolicy: common.PullPolicyIfNotPresent,
-				},
-			},
-		},
-		SystemInterrupt: make(chan os.Signal, 1),
-	}
-	build.Image.Name = common.TestDockerGitImage
-	setBuildFeatureFlag(build, featureFlagName, featureFlagValue)
-
-	abortTimer := time.AfterFunc(time.Second, func() {
-		t.Log("Interrupt")
-		build.SystemInterrupt <- os.Interrupt
-	})
-	defer abortTimer.Stop()
-
-	timeoutTimer := time.AfterFunc(time.Minute, func() {
-		t.Log("Timedout")
-		t.FailNow()
-	})
-	defer timeoutTimer.Stop()
-
-	err = build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
-	assert.EqualError(t, err, "aborted: interrupt")
-}
-
 func testKubernetesBuildCancelFeatureFlag(t *testing.T, featureFlagName string, featureFlagValue bool) {
 	if helpers.SkipIntegrationTests(t, "kubectl", "cluster-info") {
 		return
 	}
 
-	failedBuild, err := common.GetRemoteFailedBuild()
-	assert.NoError(t, err)
-	build := &common.Build{
-		JobResponse: failedBuild,
-		Runner: &common.RunnerConfig{
+	buildtest.RunBuildWithCancel(
+		t,
+		&common.RunnerConfig{
 			RunnerSettings: common.RunnerSettings{
 				Executor: "kubernetes",
 				Kubernetes: &common.KubernetesConfig{
+					Image:      common.TestAlpineImage,
 					PullPolicy: common.PullPolicyIfNotPresent,
 				},
 			},
 		},
-		SystemInterrupt: make(chan os.Signal, 1),
-	}
-	build.Image.Name = common.TestDockerGitImage
-	setBuildFeatureFlag(build, featureFlagName, featureFlagValue)
-
-	trace := &common.Trace{Writer: os.Stdout}
-
-	abortTimer := time.AfterFunc(time.Second, func() {
-		t.Log("Interrupt")
-		trace.CancelFunc()
-	})
-	defer abortTimer.Stop()
-
-	timeoutTimer := time.AfterFunc(time.Minute, func() {
-		t.Log("Timedout")
-		t.FailNow()
-	})
-	defer timeoutTimer.Stop()
-
-	err = build.Run(&common.Config{}, trace)
-	assert.IsType(t, err, &common.BuildError{})
-	assert.EqualError(t, err, "canceled")
+		func(build *common.Build) {
+			setBuildFeatureFlag(build, featureFlagName, featureFlagValue)
+		},
+	)
 }
 
 func testVolumeMountsFeatureFlag(t *testing.T, featureFlagName string, featureFlagValue bool) {
@@ -388,7 +337,12 @@ func testVolumeMountsFeatureFlag(t *testing.T, featureFlagName string, featureFl
 					Kubernetes: &common.KubernetesConfig{
 						Volumes: common.KubernetesVolumes{
 							HostPaths: []common.KubernetesHostPath{
-								{Name: "test", MountPath: "/opt/test/readonly", ReadOnly: true, HostPath: "/opt/test/rw"},
+								{
+									Name:      "test",
+									MountPath: "/opt/test/readonly",
+									ReadOnly:  true,
+									HostPath:  "/opt/test/rw",
+								},
 								{Name: "docker", MountPath: "/var/run/docker.sock"},
 							},
 							ConfigMaps: []common.KubernetesConfigMap{
@@ -428,13 +382,20 @@ func testVolumeMountsFeatureFlag(t *testing.T, featureFlagName string, featureFl
 
 			mounts := e.getVolumeMounts()
 			for _, expected := range tt.Expected {
-				assert.Contains(t, mounts, expected, "Expected volumeMount definition for %s was not found", expected.Name)
+				assert.Contains(
+					t,
+					mounts,
+					expected,
+					"Expected volumeMount definition for %s was not found",
+					expected.Name,
+				)
 			}
 		})
 	}
 }
 
 func testVolumesFeatureFlag(t *testing.T, featureFlagName string, featureFlagValue bool) {
+	//nolint:lll
 	tests := map[string]struct {
 		GlobalConfig *common.Config
 		RunnerConfig common.RunnerConfig
@@ -552,15 +513,15 @@ func testSetupBuildPodServiceCreationErrorFeatureFlag(t *testing.T, featureFlagN
 	}
 
 	fakeRoundTripper := func(req *http.Request) (*http.Response, error) {
-		body, err := ioutil.ReadAll(req.Body)
-		if !assert.NoError(t, err, "failed to read request body") {
-			return nil, err
+		body, errRT := ioutil.ReadAll(req.Body)
+		if !assert.NoError(t, errRT, "failed to read request body") {
+			return nil, errRT
 		}
 
 		p := new(api.Pod)
-		err = json.Unmarshal(body, p)
-		if !assert.NoError(t, err, "failed to read request body") {
-			return nil, err
+		errRT = json.Unmarshal(body, p)
+		if !assert.NoError(t, errRT, "failed to read request body") {
+			return nil, errRT
 		}
 
 		if req.URL.Path == "/api/v1/namespaces/default/services" {
@@ -633,7 +594,8 @@ func testKubernetesCustomClonePathFeatureFlag(t *testing.T, featureFlagName stri
 	}
 
 	jobResponse, err := common.GetRemoteBuildResponse(
-		"ls -al $CI_BUILDS_DIR/go/src/gitlab.com/gitlab-org/repo")
+		"ls -al $CI_BUILDS_DIR/go/src/gitlab.com/gitlab-org/repo",
+	)
 	require.NoError(t, err)
 
 	tests := map[string]struct {
@@ -837,6 +799,10 @@ func testInteractiveTerminalFeatureFlag(t *testing.T, featureFlagName string, fe
 		return
 	}
 
+	if os.Getenv("GITLAB_CI") == "true" {
+		t.Skip("Skipping inside of GitLab CI check https://gitlab.com/gitlab-org/gitlab-runner/-/issues/26421")
+	}
+
 	config, err := getKubeClientConfig(new(common.KubernetesConfig), new(overwrites))
 	require.NoError(t, err)
 	client, err := kubernetes.NewForConfig(config)
@@ -919,7 +885,8 @@ func TestCleanup(t *testing.T) {
 	podsEndpointURI := "/api/" + version + "/namespaces/" + objectMeta.Namespace + "/pods/" + objectMeta.Name
 	servicesEndpointURI := "/api/" + version + "/namespaces/" + objectMeta.Namespace + "/services/" + objectMeta.Name
 	secretsEndpointURI := "/api/" + version + "/namespaces/" + objectMeta.Namespace + "/secrets/" + objectMeta.Name
-	configMapsEndpointURI := "/api/" + version + "/namespaces/" + objectMeta.Namespace + "/configmaps/" + objectMeta.Name
+	configMapsEndpointURI :=
+		"/api/" + version + "/namespaces/" + objectMeta.Namespace + "/configmaps/" + objectMeta.Name
 
 	tests := []struct {
 		Name        string
@@ -1102,14 +1069,17 @@ func TestCleanup(t *testing.T) {
 
 func TestPrepare(t *testing.T) {
 	tests := []struct {
+		Name  string
+		Error string
+
 		GlobalConfig *common.Config
 		RunnerConfig *common.RunnerConfig
 		Build        *common.Build
 
 		Expected *executor
-		Error    bool
 	}{
 		{
+			Name:         "all with limits",
 			GlobalConfig: &common.Config{},
 			RunnerConfig: &common.RunnerConfig{
 				RunnerSettings: common.RunnerSettings{
@@ -1147,29 +1117,19 @@ func TestPrepare(t *testing.T) {
 					},
 				},
 				configurationOverwrites: &overwrites{
-					namespace:   "default",
-					cpuLimit:    "1.5",
-					memoryLimit: "4Gi",
+					namespace:       "default",
+					buildLimits:     mustCreateResourceList(t, "1.5", "4Gi"),
+					serviceLimits:   mustCreateResourceList(t, "100m", "200Mi"),
+					helperLimits:    mustCreateResourceList(t, "50m", "100Mi"),
+					buildRequests:   api.ResourceList{},
+					serviceRequests: api.ResourceList{},
+					helperRequests:  api.ResourceList{},
 				},
-				serviceLimits: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("100m"),
-					api.ResourceMemory: resource.MustParse("200Mi"),
-				},
-				buildLimits: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("1.5"),
-					api.ResourceMemory: resource.MustParse("4Gi"),
-				},
-				helperLimits: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("50m"),
-					api.ResourceMemory: resource.MustParse("100Mi"),
-				},
-				serviceRequests: api.ResourceList{},
-				buildRequests:   api.ResourceList{},
-				helperRequests:  api.ResourceList{},
-				pullPolicy:      "IfNotPresent",
+				pullPolicy: "IfNotPresent",
 			},
 		},
 		{
+			Name:         "all with limits and requests",
 			GlobalConfig: &common.Config{},
 			RunnerConfig: &common.RunnerConfig{
 				RunnerSettings: common.RunnerSettings{
@@ -1215,42 +1175,20 @@ func TestPrepare(t *testing.T) {
 					},
 				},
 				configurationOverwrites: &overwrites{
-					namespace:      "default",
-					serviceAccount: "not-default",
-					cpuLimit:       "1.5",
-					memoryLimit:    "4Gi",
-					cpuRequest:     "1",
-					memoryRequest:  "1.5Gi",
-				},
-				serviceLimits: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("100m"),
-					api.ResourceMemory: resource.MustParse("200Mi"),
-				},
-				buildLimits: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("1.5"),
-					api.ResourceMemory: resource.MustParse("4Gi"),
-				},
-				helperLimits: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("50m"),
-					api.ResourceMemory: resource.MustParse("100Mi"),
-				},
-				serviceRequests: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("99m"),
-					api.ResourceMemory: resource.MustParse("5Mi"),
-				},
-				buildRequests: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("1"),
-					api.ResourceMemory: resource.MustParse("1.5Gi"),
-				},
-				helperRequests: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("0.5m"),
-					api.ResourceMemory: resource.MustParse("42Mi"),
+					namespace:       "default",
+					serviceAccount:  "not-default",
+					buildLimits:     mustCreateResourceList(t, "1.5", "4Gi"),
+					buildRequests:   mustCreateResourceList(t, "1", "1.5Gi"),
+					serviceLimits:   mustCreateResourceList(t, "100m", "200Mi"),
+					serviceRequests: mustCreateResourceList(t, "99m", "5Mi"),
+					helperLimits:    mustCreateResourceList(t, "50m", "100Mi"),
+					helperRequests:  mustCreateResourceList(t, "0.5m", "42Mi"),
 				},
 			},
-			Error: false,
 		},
-
 		{
+			Name:         "unmatched service account",
+			Error:        "couldn't prepare overwrites: provided value \"not-default\" does not match \"allowed-.*\"",
 			GlobalConfig: &common.Config{},
 			RunnerConfig: &common.RunnerConfig{
 				RunnerSettings: common.RunnerSettings{
@@ -1288,47 +1226,9 @@ func TestPrepare(t *testing.T) {
 				},
 				Runner: &common.RunnerConfig{},
 			},
-			Expected: &executor{
-				options: &kubernetesOptions{
-					Image: common.Image{
-						Name: "test-image",
-					},
-				},
-				configurationOverwrites: &overwrites{
-					namespace:     "namespacee",
-					cpuLimit:      "1.5",
-					memoryLimit:   "4Gi",
-					cpuRequest:    "1",
-					memoryRequest: "1.5Gi",
-				},
-				serviceLimits: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("100m"),
-					api.ResourceMemory: resource.MustParse("200Mi"),
-				},
-				buildLimits: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("1.5"),
-					api.ResourceMemory: resource.MustParse("4Gi"),
-				},
-				helperLimits: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("50m"),
-					api.ResourceMemory: resource.MustParse("100Mi"),
-				},
-				serviceRequests: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("99m"),
-					api.ResourceMemory: resource.MustParse("5Mi"),
-				},
-				buildRequests: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("1"),
-					api.ResourceMemory: resource.MustParse("1.5Gi"),
-				},
-				helperRequests: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("0.5m"),
-					api.ResourceMemory: resource.MustParse("42Mi"),
-				},
-			},
-			Error: true,
 		},
 		{
+			Name:         "regexp match on service account and namespace",
 			GlobalConfig: &common.Config{},
 			RunnerConfig: &common.RunnerConfig{
 				RunnerSettings: common.RunnerSettings{
@@ -1363,7 +1263,7 @@ func TestPrepare(t *testing.T) {
 						Name: "test-image",
 					},
 					Variables: []common.JobVariable{
-						{Key: NamespaceOverwriteVariableName, Value: "namespacee"},
+						{Key: NamespaceOverwriteVariableName, Value: "new-namespace-name"},
 					},
 				},
 				Runner: &common.RunnerConfig{},
@@ -1375,41 +1275,19 @@ func TestPrepare(t *testing.T) {
 					},
 				},
 				configurationOverwrites: &overwrites{
-					namespace:      "namespacee",
-					serviceAccount: "a_service_account",
-					cpuLimit:       "1.5",
-					memoryLimit:    "4Gi",
-					cpuRequest:     "1",
-					memoryRequest:  "1.5Gi",
-				},
-				serviceLimits: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("100m"),
-					api.ResourceMemory: resource.MustParse("200Mi"),
-				},
-				buildLimits: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("1.5"),
-					api.ResourceMemory: resource.MustParse("4Gi"),
-				},
-				helperLimits: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("50m"),
-					api.ResourceMemory: resource.MustParse("100Mi"),
-				},
-				serviceRequests: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("99m"),
-					api.ResourceMemory: resource.MustParse("5Mi"),
-				},
-				buildRequests: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("1"),
-					api.ResourceMemory: resource.MustParse("1.5Gi"),
-				},
-				helperRequests: api.ResourceList{
-					api.ResourceCPU:    resource.MustParse("0.5m"),
-					api.ResourceMemory: resource.MustParse("42Mi"),
+					namespace:       "new-namespace-name",
+					serviceAccount:  "a_service_account",
+					buildLimits:     mustCreateResourceList(t, "1.5", "4Gi"),
+					buildRequests:   mustCreateResourceList(t, "1", "1.5Gi"),
+					serviceLimits:   mustCreateResourceList(t, "100m", "200Mi"),
+					serviceRequests: mustCreateResourceList(t, "99m", "5Mi"),
+					helperLimits:    mustCreateResourceList(t, "50m", "100Mi"),
+					helperRequests:  mustCreateResourceList(t, "0.5m", "42Mi"),
 				},
 			},
-			Error: true,
 		},
 		{
+			Name:         "regexp match on namespace",
 			GlobalConfig: &common.Config{},
 			RunnerConfig: &common.RunnerConfig{
 				RunnerSettings: common.RunnerSettings{
@@ -1440,16 +1318,19 @@ func TestPrepare(t *testing.T) {
 						Name: "test-image",
 					},
 				},
-				configurationOverwrites: &overwrites{namespace: "namespace-0"},
-				serviceLimits:           api.ResourceList{},
-				buildLimits:             api.ResourceList{},
-				helperLimits:            api.ResourceList{},
-				serviceRequests:         api.ResourceList{},
-				buildRequests:           api.ResourceList{},
-				helperRequests:          api.ResourceList{},
+				configurationOverwrites: &overwrites{
+					namespace:       "namespace-0",
+					serviceLimits:   api.ResourceList{},
+					buildLimits:     api.ResourceList{},
+					helperLimits:    api.ResourceList{},
+					serviceRequests: api.ResourceList{},
+					buildRequests:   api.ResourceList{},
+					helperRequests:  api.ResourceList{},
+				},
 			},
 		},
 		{
+			Name:         "minimal configuration",
 			GlobalConfig: &common.Config{},
 			RunnerConfig: &common.RunnerConfig{
 				RunnerSettings: common.RunnerSettings{
@@ -1473,16 +1354,19 @@ func TestPrepare(t *testing.T) {
 						Name: "test-image",
 					},
 				},
-				configurationOverwrites: &overwrites{namespace: "default"},
-				serviceLimits:           api.ResourceList{},
-				buildLimits:             api.ResourceList{},
-				helperLimits:            api.ResourceList{},
-				serviceRequests:         api.ResourceList{},
-				buildRequests:           api.ResourceList{},
-				helperRequests:          api.ResourceList{},
+				configurationOverwrites: &overwrites{
+					namespace:       "default",
+					serviceLimits:   api.ResourceList{},
+					buildLimits:     api.ResourceList{},
+					helperLimits:    api.ResourceList{},
+					serviceRequests: api.ResourceList{},
+					buildRequests:   api.ResourceList{},
+					helperRequests:  api.ResourceList{},
+				},
 			},
 		},
 		{
+			Name:         "image and one service",
 			GlobalConfig: &common.Config{},
 			RunnerConfig: &common.RunnerConfig{
 				RunnerSettings: common.RunnerSettings{
@@ -1524,16 +1408,19 @@ func TestPrepare(t *testing.T) {
 						},
 					},
 				},
-				configurationOverwrites: &overwrites{namespace: "default"},
-				serviceLimits:           api.ResourceList{},
-				buildLimits:             api.ResourceList{},
-				helperLimits:            api.ResourceList{},
-				serviceRequests:         api.ResourceList{},
-				buildRequests:           api.ResourceList{},
-				helperRequests:          api.ResourceList{},
+				configurationOverwrites: &overwrites{
+					namespace:       "default",
+					serviceLimits:   api.ResourceList{},
+					buildLimits:     api.ResourceList{},
+					helperLimits:    api.ResourceList{},
+					serviceRequests: api.ResourceList{},
+					buildRequests:   api.ResourceList{},
+					helperRequests:  api.ResourceList{},
+				},
 			},
 		},
 		{
+			Name:         "merge services",
 			GlobalConfig: &common.Config{},
 			RunnerConfig: &common.RunnerConfig{
 				RunnerSettings: common.RunnerSettings{
@@ -1589,19 +1476,21 @@ func TestPrepare(t *testing.T) {
 						},
 					},
 				},
-				configurationOverwrites: &overwrites{namespace: "default"},
-				serviceLimits:           api.ResourceList{},
-				buildLimits:             api.ResourceList{},
-				helperLimits:            api.ResourceList{},
-				serviceRequests:         api.ResourceList{},
-				buildRequests:           api.ResourceList{},
-				helperRequests:          api.ResourceList{},
+				configurationOverwrites: &overwrites{
+					namespace:       "default",
+					serviceLimits:   api.ResourceList{},
+					buildLimits:     api.ResourceList{},
+					helperLimits:    api.ResourceList{},
+					serviceRequests: api.ResourceList{},
+					buildRequests:   api.ResourceList{},
+					helperRequests:  api.ResourceList{},
+				},
 			},
 		},
 	}
 
-	for index, test := range tests {
-		t.Run(strconv.Itoa(index), func(t *testing.T) {
+	for _, test := range tests {
+		t.Run(test.Name, func(t *testing.T) {
 			e := &executor{
 				AbstractExecutor: executors.AbstractExecutor{
 					ExecutorOptions: executorOptions,
@@ -1615,17 +1504,12 @@ func TestPrepare(t *testing.T) {
 			}
 
 			err := e.Prepare(prepareOptions)
-
 			if err != nil {
 				assert.False(t, test.Build.IsSharedEnv())
-				if test.Error {
-					assert.Error(t, err)
-				} else {
-					assert.NoError(t, err)
-				}
-				if !test.Error {
-					t.Errorf("Got error. Expected: %v", test.Expected)
-				}
+			}
+			if test.Error != "" {
+				assert.Error(t, err)
+				assert.Contains(t, err.Error(), test.Error)
 				return
 			}
 
@@ -1639,6 +1523,8 @@ func TestPrepare(t *testing.T) {
 			e.kubeClient = nil
 			e.kubeConfig = nil
 			e.featureChecker = nil
+
+			assert.NoError(t, err)
 			assert.Equal(t, test.Expected, e)
 		})
 	}
@@ -2411,7 +2297,11 @@ func TestSetupBuildPod(t *testing.T) {
 			VerifyExecutorFn: func(t *testing.T, test setupBuildPodTestDef, e *executor) {
 				assert.Equal(t, "servicename-non-compatble", e.services[0].GenerateName)
 				assert.NotEmpty(t, e.ProxyPool["service,name-.non-compat!ble"])
-				assert.Equal(t, "port,name-.non-compat!ble", e.ProxyPool["service,name-.non-compat!ble"].Settings.Ports[0].Name)
+				assert.Equal(
+					t,
+					"port,name-.non-compat!ble",
+					e.ProxyPool["service,name-.non-compat!ble"].Settings.Ports[0].Name,
+				)
 			},
 		},
 		"sets command (entrypoint) and args": {
@@ -2522,6 +2412,108 @@ func TestSetupBuildPod(t *testing.T) {
 				assert.Empty(t, pod.Spec.SecurityContext, "Security context should be empty")
 			},
 		},
+		"supports pod node affinities": {
+			RunnerConfig: common.RunnerConfig{
+				RunnerSettings: common.RunnerSettings{
+					Kubernetes: &common.KubernetesConfig{
+						Namespace: "default",
+						Affinity: common.KubernetesAffinity{
+							NodeAffinity: &common.KubernetesNodeAffinity{
+								PreferredDuringSchedulingIgnoredDuringExecution: []common.PreferredSchedulingTerm{
+									{
+										Weight: 100,
+										Preference: common.NodeSelectorTerm{
+											MatchExpressions: []common.NodeSelectorRequirement{
+												{
+													Key:      "cpu_speed",
+													Operator: "In",
+													Values:   []string{"fast"},
+												},
+											},
+											MatchFields: []common.NodeSelectorRequirement{
+												{
+													Key:      "cpu_count",
+													Operator: "Gt",
+													Values:   []string{"12"},
+												},
+											},
+										},
+									},
+									{
+										Weight: 50,
+										Preference: common.NodeSelectorTerm{
+											MatchExpressions: []common.NodeSelectorRequirement{
+												{
+													Key:      "kubernetes.io/e2e-az-name",
+													Operator: "In",
+													Values:   []string{"e2e-az1", "e2e-az2"},
+												},
+												{
+													Key:      "kubernetes.io/arch",
+													Operator: "NotIn",
+													Values:   []string{"arm"},
+												},
+											},
+										},
+									},
+								},
+								RequiredDuringSchedulingIgnoredDuringExecution: &common.NodeSelector{
+									NodeSelectorTerms: []common.NodeSelectorTerm{
+										{
+											MatchExpressions: []common.NodeSelectorRequirement{
+												{
+													Key:      "kubernetes.io/e2e-az-name",
+													Operator: "In",
+													Values:   []string{"e2e-az1", "e2e-az2"},
+												},
+											},
+										},
+									},
+								},
+							},
+						},
+					},
+				},
+			},
+			//nolint:lll
+			VerifyFn: func(t *testing.T, test setupBuildPodTestDef, pod *api.Pod) {
+				require.NotNil(t, pod.Spec.Affinity)
+				require.NotNil(t, pod.Spec.Affinity.NodeAffinity)
+
+				nodeAffinity := pod.Spec.Affinity.NodeAffinity
+				preferredNodeAffinity := nodeAffinity.PreferredDuringSchedulingIgnoredDuringExecution
+
+				require.Len(t, preferredNodeAffinity, 2)
+				assert.Equal(t, int32(100), preferredNodeAffinity[0].Weight)
+				require.Len(t, preferredNodeAffinity[0].Preference.MatchExpressions, 1)
+				require.Len(t, preferredNodeAffinity[0].Preference.MatchFields, 1)
+				assert.Equal(t, "cpu_speed", preferredNodeAffinity[0].Preference.MatchExpressions[0].Key)
+				assert.Equal(t, api.NodeSelectorOperator("In"), preferredNodeAffinity[0].Preference.MatchExpressions[0].Operator)
+				assert.Equal(t, []string{"fast"}, preferredNodeAffinity[0].Preference.MatchExpressions[0].Values)
+				assert.Equal(t, "cpu_count", preferredNodeAffinity[0].Preference.MatchFields[0].Key)
+				assert.Equal(t, api.NodeSelectorOperator("Gt"), preferredNodeAffinity[0].Preference.MatchFields[0].Operator)
+				assert.Equal(t, []string{"12"}, preferredNodeAffinity[0].Preference.MatchFields[0].Values)
+
+				assert.Equal(t, int32(50), preferredNodeAffinity[1].Weight)
+				require.Len(t, preferredNodeAffinity[1].Preference.MatchExpressions, 2)
+				assert.Equal(t, "kubernetes.io/e2e-az-name", preferredNodeAffinity[1].Preference.MatchExpressions[0].Key)
+				assert.Equal(t, api.NodeSelectorOperator("In"), preferredNodeAffinity[1].Preference.MatchExpressions[0].Operator)
+				assert.Equal(t, []string{"e2e-az1", "e2e-az2"}, preferredNodeAffinity[1].Preference.MatchExpressions[0].Values)
+				assert.Equal(t, "kubernetes.io/arch", preferredNodeAffinity[1].Preference.MatchExpressions[1].Key)
+				assert.Equal(t, api.NodeSelectorOperator("NotIn"), preferredNodeAffinity[1].Preference.MatchExpressions[1].Operator)
+				assert.Equal(t, []string{"arm"}, preferredNodeAffinity[1].Preference.MatchExpressions[1].Values)
+
+				require.NotNil(t, nodeAffinity.RequiredDuringSchedulingIgnoredDuringExecution)
+				requiredNodeAffinity := nodeAffinity.RequiredDuringSchedulingIgnoredDuringExecution
+
+				require.Len(t, requiredNodeAffinity.NodeSelectorTerms, 1)
+				require.Len(t, requiredNodeAffinity.NodeSelectorTerms[0].MatchExpressions, 1)
+				require.Len(t, requiredNodeAffinity.NodeSelectorTerms[0].MatchFields, 0)
+				assert.Equal(t, "kubernetes.io/e2e-az-name", requiredNodeAffinity.NodeSelectorTerms[0].MatchExpressions[0].Key)
+				assert.Equal(t, api.NodeSelectorOperator("In"), requiredNodeAffinity.NodeSelectorTerms[0].MatchExpressions[0].Operator)
+				assert.Equal(t, []string{"e2e-az1", "e2e-az2"}, requiredNodeAffinity.NodeSelectorTerms[0].MatchExpressions[0].Values)
+			},
+		},
 		"supports services as host aliases": {
 			RunnerConfig: common.RunnerConfig{
 				RunnerSettings: common.RunnerSettings{
@@ -3000,65 +2992,6 @@ func TestRunAttachCheckPodStatus(t *testing.T) {
 	}
 }
 
-func TestLimits(t *testing.T) {
-	tests := []struct {
-		CPU, Memory string
-		Expected    api.ResourceList
-		ExpectedErr error
-	}{
-		{
-			CPU:    "100m",
-			Memory: "100Mi",
-			Expected: api.ResourceList{
-				api.ResourceCPU:    resource.MustParse("100m"),
-				api.ResourceMemory: resource.MustParse("100Mi"),
-			},
-			ExpectedErr: nil,
-		},
-		{
-			CPU: "100m",
-			Expected: api.ResourceList{
-				api.ResourceCPU: resource.MustParse("100m"),
-			},
-			ExpectedErr: nil,
-		},
-		{
-			Memory: "100Mi",
-			Expected: api.ResourceList{
-				api.ResourceMemory: resource.MustParse("100Mi"),
-			},
-			ExpectedErr: nil,
-		},
-		{
-			CPU:         "100j",
-			Expected:    api.ResourceList{},
-			ExpectedErr: resource.ErrFormatWrong,
-		},
-		{
-			Memory:      "100j",
-			Expected:    api.ResourceList{},
-			ExpectedErr: resource.ErrFormatWrong,
-		},
-		{
-			Expected:    api.ResourceList{},
-			ExpectedErr: nil,
-		},
-	}
-
-	for _, tc := range tests {
-		t.Run(fmt.Sprintf("CPU=%s/Memory=%s", tc.CPU, tc.Memory), func(t *testing.T) {
-			res, err := limits(tc.CPU, tc.Memory)
-			assert.True(t,
-				errors.Is(err, tc.ExpectedErr),
-				"expected err %T, but got %T",
-				tc.ExpectedErr,
-				err,
-			)
-			assert.Equal(t, tc.Expected, res)
-		})
-	}
-}
-
 func fakeConfigMap() *api.ConfigMap {
 	configMap := &api.ConfigMap{}
 	configMap.Name = "fake"
@@ -3125,7 +3058,9 @@ func TestNewLogStreamerStream(t *testing.T) {
 
 		return true
 	})
-	remoteExecutor.On("Execute", http.MethodPost, urlMatcher, mock.Anything, nil, output, output, false).Return(abortErr)
+	remoteExecutor.
+		On("Execute", http.MethodPost, urlMatcher, mock.Anything, nil, output, output, false).
+		Return(abortErr)
 
 	p, ok := e.newLogProcessor().(*kubernetesLogProcessor)
 	require.True(t, ok)
@@ -3159,6 +3094,7 @@ func (f FakeBuildTrace) Success()                                              {
 func (f FakeBuildTrace) Fail(err error, failureReason common.JobFailureReason) {}
 func (f FakeBuildTrace) Notify(func())                                         {}
 func (f FakeBuildTrace) SetCancelFunc(cancelFunc context.CancelFunc)           {}
+func (f FakeBuildTrace) Cancel() bool                                          { return false }
 func (f FakeBuildTrace) SetFailuresCollector(fc common.FailuresCollector)      {}
 func (f FakeBuildTrace) SetMasked(masked []string)                             {}
 func (f FakeBuildTrace) IsStdout() bool {
@@ -3187,7 +3123,12 @@ func TestCommandTerminatedError_Is(t *testing.T) {
 
 	for tn, tt := range tests {
 		t.Run(tn, func(t *testing.T) {
-			assert.Equal(t, tt.expectedIsResult, errors.Is(tt.err, new(commandTerminatedError)), "commandTerminatedError.Is incorrect result")
+			assert.Equal(
+				t,
+				tt.expectedIsResult,
+				errors.Is(tt.err, new(commandTerminatedError)),
+				"commandTerminatedError.Is incorrect result",
+			)
 		})
 	}
 }
