@@ -20,6 +20,21 @@ import (
 
 type kubeConfigProvider func() (*restclient.Config, error)
 
+type resourceQuantityError struct {
+	resource string
+	value    string
+	inner    error
+}
+
+func (r *resourceQuantityError) Error() string {
+	return fmt.Sprintf("parsing resource %q with value %q: %q", r.resource, r.value, r.inner)
+}
+
+func (r *resourceQuantityError) Is(err error) bool {
+	t, ok := err.(*resourceQuantityError)
+	return ok && r.resource == t.resource && r.value == t.value && r.inner == t.inner
+}
+
 var (
 	// inClusterConfig parses kubernets configuration reading in cluster values
 	inClusterConfig kubeConfigProvider = restclient.InClusterConfig
@@ -27,7 +42,10 @@ var (
 	defaultKubectlConfig kubeConfigProvider = loadDefaultKubectlConfig
 )
 
-func getKubeClientConfig(config *common.KubernetesConfig, overwrites *overwrites) (kubeConfig *restclient.Config, err error) {
+func getKubeClientConfig(
+	config *common.KubernetesConfig,
+	overwrites *overwrites,
+) (kubeConfig *restclient.Config, err error) {
 	if len(config.Host) > 0 {
 		kubeConfig, err = getOutClusterClientConfig(config)
 	} else {
@@ -37,8 +55,8 @@ func getKubeClientConfig(config *common.KubernetesConfig, overwrites *overwrites
 		return nil, err
 	}
 
-	//apply overwrites
-	if len(overwrites.bearerToken) > 0 {
+	// apply overwrites
+	if overwrites.bearerToken != "" {
 		kubeConfig.BearerToken = overwrites.bearerToken
 	}
 
@@ -57,8 +75,8 @@ func getOutClusterClientConfig(config *common.KubernetesConfig) (*restclient.Con
 	}
 
 	// certificate based auth
-	if len(config.CertFile) > 0 {
-		if len(config.KeyFile) == 0 || len(config.CAFile) == 0 {
+	if config.CertFile != "" {
+		if config.KeyFile == "" || config.CAFile == "" {
 			return nil, fmt.Errorf("ca file, cert file and key file must be specified when using file based auth")
 		}
 
@@ -148,14 +166,20 @@ func getPodPhase(c *kubernetes.Clientset, pod *api.Pod, out io.Writer) podPhaseR
 		}
 
 		switch container.State.Waiting.Reason {
-		case "ErrImagePull", "ImagePullBackOff":
+		case "ErrImagePull", "ImagePullBackOff", "InvalidImageName":
 			err = fmt.Errorf("image pull failed: %s", container.State.Waiting.Message)
 			err = &common.BuildError{Inner: err}
 			return podPhaseResponse{true, api.PodUnknown, err}
 		}
 	}
 
-	fmt.Fprintf(out, "Waiting for pod %s/%s to be running, status is %s\n", pod.Namespace, pod.Name, pod.Status.Phase)
+	_, _ = fmt.Fprintf(
+		out,
+		"Waiting for pod %s/%s to be running, status is %s\n",
+		pod.Namespace,
+		pod.Name,
+		pod.Status.Phase,
+	)
 	return podPhaseResponse{false, pod.Status.Phase, nil}
 }
 
@@ -176,7 +200,13 @@ func triggerPodPhaseCheck(c *kubernetes.Clientset, pod *api.Pod, out io.Writer)
 // reached.
 // The timeout and polling values are configurable through KubernetesConfig
 // parameters.
-func waitForPodRunning(ctx context.Context, c *kubernetes.Clientset, pod *api.Pod, out io.Writer, config *common.KubernetesConfig) (api.PodPhase, error) {
+func waitForPodRunning(
+	ctx context.Context,
+	c *kubernetes.Clientset,
+	pod *api.Pod,
+	out io.Writer,
+	config *common.KubernetesConfig,
+) (api.PodPhase, error) {
 	pollInterval := config.GetPollInterval()
 	pollAttempts := config.GetPollAttempts()
 	for i := 0; i <= pollAttempts; i++ {
@@ -198,27 +228,27 @@ func waitForPodRunning(ctx context.Context, c *kubernetes.Clientset, pod *api.Po
 // and returns a ResourceList with appropriately scaled Quantity
 // values for Kubernetes. This allows users to write "500m" for CPU,
 // and "50Mi" for memory (etc.)
-func limits(cpu, memory string) (api.ResourceList, error) {
+func createResourceList(cpu, memory string) (api.ResourceList, error) {
 	var rCPU, rMem resource.Quantity
 	var err error
 
 	parse := func(s string) (resource.Quantity, error) {
 		var q resource.Quantity
-		if len(s) == 0 {
+		if s == "" {
 			return q, nil
 		}
 		if q, err = resource.ParseQuantity(s); err != nil {
-			return q, fmt.Errorf("parsing resource limit: %w", err)
+			return q, err
 		}
 		return q, nil
 	}
 
 	if rCPU, err = parse(cpu); err != nil {
-		return api.ResourceList{}, err
+		return api.ResourceList{}, &resourceQuantityError{resource: "cpu", value: cpu, inner: err}
 	}
 
 	if rMem, err = parse(memory); err != nil {
-		return api.ResourceList{}, err
+		return api.ResourceList{}, &resourceQuantityError{resource: "memory", value: memory, inner: err}
 	}
 
 	l := make(api.ResourceList)
