@@ -23,6 +23,7 @@ import (
 	"gitlab.com/gitlab-org/gitlab-runner/common"
 	"gitlab.com/gitlab-org/gitlab-runner/common/buildtest"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers"
+	"gitlab.com/gitlab-org/gitlab-runner/helpers/test"
 	"gitlab.com/gitlab-org/gitlab-runner/session"
 	"gitlab.com/gitlab-org/gitlab-runner/shells/shellstest"
 )
@@ -48,7 +49,10 @@ func skipOnGitWithMessage(t *testing.T, constraints string, message string) {
 		return
 	}
 
-	gitVersion, err := version.NewVersion(strings.TrimSpace(split[2]))
+	// Take the first 3 components of the version (so we ignore suffix in e.g. 2.23.0.windows.1)
+	versionComponents := strings.Split(strings.TrimSpace(split[2]), ".")
+	versionStr := strings.Join(versionComponents[:3], ".")
+	gitVersion, err := version.NewVersion(versionStr)
 	if err != nil {
 		t.Fatal("Can't detect git version", err)
 		return
@@ -205,6 +209,9 @@ func TestRawVariableOutput(t *testing.T) {
 		"powershell": {
 			command: "echo $env:TEST",
 		},
+		"pwsh": {
+			command: "echo $env:TEST",
+		},
 	}
 
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
@@ -232,42 +239,23 @@ func TestRawVariableOutput(t *testing.T) {
 	})
 }
 
-func TestBuildAbort(t *testing.T) {
-	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
-		longRunningBuild, err := common.GetLongRunningBuild()
-		assert.NoError(t, err)
-		build, cleanup := newBuild(t, longRunningBuild, shell)
-		defer cleanup()
-
-		abortTimer := time.AfterFunc(time.Second, func() {
-			t.Log("Interrupt")
-			build.SystemInterrupt <- os.Interrupt
-		})
-		defer abortTimer.Stop()
-
-		err = buildtest.RunBuild(t, build)
-		assert.EqualError(t, err, "aborted: interrupt")
-	})
-}
-
 func TestBuildCancel(t *testing.T) {
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
-		longRunningBuild, err := common.GetLongRunningBuild()
-		assert.NoError(t, err)
-		build, cleanup := newBuild(t, longRunningBuild, shell)
+		build, cleanup := newBuild(t, common.JobResponse{}, shell)
 		defer cleanup()
 
-		trace := &common.Trace{Writer: os.Stdout}
+		updateSleepForCMD := func(build *common.Build) {
+			if shell != "cmd" {
+				return
+			}
 
-		cancelTimer := time.AfterFunc(time.Second, func() {
-			t.Log("Cancel")
-			trace.CancelFunc()
-		})
-		defer cancelTimer.Stop()
+			resp, err := common.GetRemoteLongRunningBuildWithAfterScriptCMD()
+			require.NoError(t, err)
 
-		err = buildtest.RunBuildWithTrace(t, build, trace)
-		assert.EqualError(t, err, "canceled")
-		assert.IsType(t, err, &common.BuildError{})
+			build.JobResponse = resp
+		}
+
+		buildtest.RunBuildWithCancel(t, build.Runner, updateSleepForCMD)
 	})
 }
 
@@ -297,9 +285,11 @@ func TestBuildWithShallowLock(t *testing.T) {
 		build, cleanup := newBuild(t, successfulBuild, shell)
 		defer cleanup()
 
-		build.Variables = append(build.Variables,
+		build.Variables = append(
+			build.Variables,
 			common.JobVariable{Key: "GIT_DEPTH", Value: "1"},
-			common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"})
+			common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"},
+		)
 
 		err = buildtest.RunBuild(t, build)
 		assert.NoError(t, err)
@@ -365,7 +355,12 @@ func assertLFSFileNotDownloaded(t *testing.T, build *common.Build) {
 	lfsFilePath := filepath.Join(build.FullProjectDir(), "files", "lfs", "file_1.lfs")
 	info, err := os.Stat(lfsFilePath)
 	require.NoError(t, err)
-	assert.True(t, info.Size() < common.FilesLFSFile1LFSsize, "invalid size of %q file - expected to be less then downloaded LFS object", lfsFilePath)
+	assert.True(
+		t,
+		info.Size() < common.FilesLFSFile1LFSsize,
+		"invalid size of %q file - expected to be less then downloaded LFS object",
+		lfsFilePath,
+	)
 }
 
 func assertLFSFileNotPresent(t *testing.T, build *common.Build) {
@@ -473,8 +468,11 @@ func TestBuildWithGitStrategyFetchWithUserDisabledLFS(t *testing.T) {
 		build, cleanup := newBuild(t, successfulBuild, shell)
 		defer cleanup()
 
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_LFS_SKIP_SMUDGE", Value: "1", Public: true})
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"})
+		build.Variables = append(
+			build.Variables,
+			common.JobVariable{Key: "GIT_LFS_SKIP_SMUDGE", Value: "1", Public: true},
+			common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"},
+		)
 
 		out, err := buildtest.RunBuildReturningOutput(t, build)
 		assert.NoError(t, err)
@@ -483,7 +481,10 @@ func TestBuildWithGitStrategyFetchWithUserDisabledLFS(t *testing.T) {
 		assertLFSFileNotPresent(t, build)
 
 		build.GitInfo = common.GetLFSGitInfo(build.GitInfo.RepoURL)
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_LFS_SKIP_SMUDGE", Value: "1", Public: true})
+		build.Variables = append(
+			build.Variables,
+			common.JobVariable{Key: "GIT_LFS_SKIP_SMUDGE", Value: "1", Public: true},
+		)
 
 		out, err = buildtest.RunBuildReturningOutput(t, build)
 		assert.NoError(t, err)
@@ -501,8 +502,11 @@ func TestBuildWithGitStrategyFetchNoCheckoutWithoutLFS(t *testing.T) {
 		defer cleanup()
 
 		build.Runner.PreCloneScript = "echo pre-clone-script"
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"})
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_CHECKOUT", Value: "false"})
+		build.Variables = append(
+			build.Variables,
+			common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"},
+			common.JobVariable{Key: "GIT_CHECKOUT", Value: "false"},
+		)
 
 		out, err := buildtest.RunBuildReturningOutput(t, build)
 		assert.NoError(t, err)
@@ -526,8 +530,11 @@ func TestBuildWithGitStrategyFetchNoCheckoutWithLFS(t *testing.T) {
 		build, cleanup := newBuild(t, successfulBuild, shell)
 		defer cleanup()
 
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"})
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_CHECKOUT", Value: "false"})
+		build.Variables = append(
+			build.Variables,
+			common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"},
+			common.JobVariable{Key: "GIT_CHECKOUT", Value: "false"},
+		)
 
 		out, err := buildtest.RunBuildReturningOutput(t, build)
 		assert.NoError(t, err)
@@ -594,8 +601,11 @@ func TestBuildWithGitStrategyCloneWithUserDisabledLFS(t *testing.T) {
 		build, cleanup := newBuild(t, successfulBuild, shell)
 		defer cleanup()
 
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_STRATEGY", Value: "clone"})
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_LFS_SKIP_SMUDGE", Value: "1", Public: true})
+		build.Variables = append(
+			build.Variables,
+			common.JobVariable{Key: "GIT_STRATEGY", Value: "clone"},
+			common.JobVariable{Key: "GIT_LFS_SKIP_SMUDGE", Value: "1", Public: true},
+		)
 
 		out, err := buildtest.RunBuildReturningOutput(t, build)
 		assert.NoError(t, err)
@@ -612,8 +622,11 @@ func TestBuildWithGitStrategyCloneNoCheckoutWithoutLFS(t *testing.T) {
 		defer cleanup()
 
 		build.Runner.PreCloneScript = "echo pre-clone-script"
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_STRATEGY", Value: "clone"})
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_CHECKOUT", Value: "false"})
+		build.Variables = append(
+			build.Variables,
+			common.JobVariable{Key: "GIT_STRATEGY", Value: "clone"},
+			common.JobVariable{Key: "GIT_CHECKOUT", Value: "false"},
+		)
 
 		out, err := buildtest.RunBuildReturningOutput(t, build)
 		assert.NoError(t, err)
@@ -636,8 +649,11 @@ func TestBuildWithGitStrategyCloneNoCheckoutWithLFS(t *testing.T) {
 		build, cleanup := newBuild(t, successfulBuild, shell)
 		defer cleanup()
 
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_STRATEGY", Value: "clone"})
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_CHECKOUT", Value: "false"})
+		build.Variables = append(
+			build.Variables,
+			common.JobVariable{Key: "GIT_STRATEGY", Value: "clone"},
+			common.JobVariable{Key: "GIT_CHECKOUT", Value: "false"},
+		)
 
 		out, err := buildtest.RunBuildReturningOutput(t, build)
 		assert.NoError(t, err)
@@ -655,8 +671,11 @@ func TestBuildWithSubmoduleLFSPullsLFSObject(t *testing.T) {
 		assert.NoError(t, err)
 		build, cleanup := newBuild(t, successfulBuild, shell)
 		defer cleanup()
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"})
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_SUBMODULE_STRATEGY", Value: "normal"})
+		build.Variables = append(
+			build.Variables,
+			common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"},
+			common.JobVariable{Key: "GIT_SUBMODULE_STRATEGY", Value: "normal"},
+		)
 		build.GitInfo = common.GetSubmoduleLFSGitInfo(build.GitInfo.RepoURL)
 
 		out, err := buildtest.RunBuildReturningOutput(t, build)
@@ -677,9 +696,12 @@ func TestBuildWithSubmoduleLFSDisabledSmudging(t *testing.T) {
 		assert.NoError(t, err)
 		build, cleanup := newBuild(t, successfulBuild, shell)
 		defer cleanup()
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"})
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_SUBMODULE_STRATEGY", Value: "normal"})
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_LFS_SKIP_SMUDGE", Value: "1", Public: true})
+		build.Variables = append(
+			build.Variables,
+			common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"},
+			common.JobVariable{Key: "GIT_SUBMODULE_STRATEGY", Value: "normal"},
+			common.JobVariable{Key: "GIT_LFS_SKIP_SMUDGE", Value: "1", Public: true},
+		)
 		build.GitInfo = common.GetSubmoduleLFSGitInfo(build.GitInfo.RepoURL)
 
 		out, err := buildtest.RunBuildReturningOutput(t, build)
@@ -701,7 +723,10 @@ func TestBuildWithGitSubmoduleStrategyNone(t *testing.T) {
 				build, cleanup := newBuild(t, successfulBuild, shell)
 				defer cleanup()
 
-				build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_SUBMODULE_STRATEGY", Value: "none"})
+				build.Variables = append(
+					build.Variables,
+					common.JobVariable{Key: "GIT_SUBMODULE_STRATEGY", Value: "none"},
+				)
 
 				out, err := buildtest.RunBuildReturningOutput(t, build)
 				assert.NoError(t, err)
@@ -776,8 +801,11 @@ func TestBuildWithGitFetchSubmoduleStrategyRecursive(t *testing.T) {
 		build, cleanup := newBuild(t, successfulBuild, shell)
 		defer cleanup()
 
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"})
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_SUBMODULE_STRATEGY", Value: "recursive"})
+		build.Variables = append(
+			build.Variables,
+			common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"},
+			common.JobVariable{Key: "GIT_SUBMODULE_STRATEGY", Value: "recursive"},
+		)
 
 		out, err := buildtest.RunBuildReturningOutput(t, build)
 		assert.NoError(t, err)
@@ -828,8 +856,11 @@ func TestBuildWithGitSubmoduleStrategyRecursiveAndGitStrategyNone(t *testing.T)
 		build, cleanup := newBuild(t, successfulBuild, shell)
 		defer cleanup()
 
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_STRATEGY", Value: "none"})
-		build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_SUBMODULE_STRATEGY", Value: "recursive"})
+		build.Variables = append(
+			build.Variables,
+			common.JobVariable{Key: "GIT_STRATEGY", Value: "none"},
+			common.JobVariable{Key: "GIT_SUBMODULE_STRATEGY", Value: "recursive"},
+		)
 
 		out, err := buildtest.RunBuildReturningOutput(t, build)
 		assert.NoError(t, err)
@@ -924,6 +955,8 @@ func TestBuildWithDebugTrace(t *testing.T) {
 }
 
 func TestBuildMultilineCommand(t *testing.T) {
+	test.SkipIfGitLabCIOn(t, test.OSWindows)
+
 	multilineBuild, err := common.GetMultilineBashBuild()
 	assert.NoError(t, err)
 	build, cleanup := newBuild(t, multilineBuild, "bash")
@@ -1038,40 +1071,53 @@ func TestBuildChangesBranchesWhenFetchingRepo(t *testing.T) {
 }
 
 func TestBuildPowerShellCatchesExceptions(t *testing.T) {
-	if helpers.SkipIntegrationTests(t, "powershell") {
-		t.Skip()
+	for _, shell := range []string{"powershell", "pwsh"} {
+		t.Run(shell, func(t *testing.T) {
+			helpers.SkipIntegrationTests(t, shell)
+
+			successfulBuild, err := common.GetRemoteSuccessfulBuild()
+			assert.NoError(t, err)
+			build, cleanup := newBuild(t, successfulBuild, shell)
+			defer cleanup()
+			build.Variables = append(
+				build.Variables,
+				common.JobVariable{Key: "ErrorActionPreference", Value: "Stop"},
+				common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"},
+			)
+
+			out, err := buildtest.RunBuildReturningOutput(t, build)
+			assert.NoError(t, err)
+			assert.Contains(t, out, "Created fresh repository")
+
+			out, err = buildtest.RunBuildReturningOutput(t, build)
+			assert.NoError(t, err)
+			assert.NotContains(t, out, "Created fresh repository")
+			assert.Regexp(t, "Checking out [a-f0-9]+ as", out)
+
+			build.Variables = append(
+				build.Variables,
+				common.JobVariable{Key: "ErrorActionPreference", Value: "Continue"},
+			)
+			out, err = buildtest.RunBuildReturningOutput(t, build)
+			assert.NoError(t, err)
+			assert.NotContains(t, out, "Created fresh repository")
+			assert.Regexp(t, "Checking out [a-f0-9]+ as", out)
+
+			build.Variables = append(
+				build.Variables,
+				common.JobVariable{Key: "ErrorActionPreference", Value: "SilentlyContinue"},
+			)
+			out, err = buildtest.RunBuildReturningOutput(t, build)
+			assert.NoError(t, err)
+			assert.NotContains(t, out, "Created fresh repository")
+			assert.Regexp(t, "Checking out [a-f0-9]+ as", out)
+		})
 	}
-
-	successfulBuild, err := common.GetRemoteSuccessfulBuild()
-	assert.NoError(t, err)
-	build, cleanup := newBuild(t, successfulBuild, "powershell")
-	defer cleanup()
-	build.Variables = append(build.Variables, common.JobVariable{Key: "ErrorActionPreference", Value: "Stop"})
-	build.Variables = append(build.Variables, common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"})
-
-	out, err := buildtest.RunBuildReturningOutput(t, build)
-	assert.NoError(t, err)
-	assert.Contains(t, out, "Created fresh repository")
-
-	out, err = buildtest.RunBuildReturningOutput(t, build)
-	assert.NoError(t, err)
-	assert.NotContains(t, out, "Created fresh repository")
-	assert.Regexp(t, "Checking out [a-f0-9]+ as", out)
-
-	build.Variables = append(build.Variables, common.JobVariable{Key: "ErrorActionPreference", Value: "Continue"})
-	out, err = buildtest.RunBuildReturningOutput(t, build)
-	assert.NoError(t, err)
-	assert.NotContains(t, out, "Created fresh repository")
-	assert.Regexp(t, "Checking out [a-f0-9]+ as", out)
-
-	build.Variables = append(build.Variables, common.JobVariable{Key: "ErrorActionPreference", Value: "SilentlyContinue"})
-	out, err = buildtest.RunBuildReturningOutput(t, build)
-	assert.NoError(t, err)
-	assert.NotContains(t, out, "Created fresh repository")
-	assert.Regexp(t, "Checking out [a-f0-9]+ as", out)
 }
 
 func TestInteractiveTerminal(t *testing.T) {
+	test.SkipIfGitLabCIOn(t, test.OSWindows)
+
 	cases := []struct {
 		app                string
 		shell              string
@@ -1116,13 +1162,13 @@ func TestInteractiveTerminal(t *testing.T) {
 
 			go func() {
 				buf := bytes.NewBuffer(nil)
-				err := buildtest.RunBuildWithOptions(
+				errRun := buildtest.RunBuildWithOptions(
 					t,
 					build,
 					&common.Trace{Writer: buf},
 					&common.Config{SessionServer: common.SessionServer{SessionTimeout: 2}},
 				)
-				require.NoError(t, err)
+				require.NoError(t, errRun)
 
 				buildOut <- buf.String()
 			}()
@@ -1144,8 +1190,9 @@ func TestInteractiveTerminal(t *testing.T) {
 				"Authorization": []string{build.Session.Token},
 			}
 			conn, resp, err := websocket.DefaultDialer.Dial(u.String(), headers)
-			assert.NoError(t, err)
 			assert.Equal(t, c.expectedStatusCode, resp.StatusCode)
+			body, _ := ioutil.ReadAll(resp.Body)
+			require.NoError(t, err, string(body))
 			defer resp.Body.Close()
 
 			defer func() {
@@ -1160,14 +1207,14 @@ func TestInteractiveTerminal(t *testing.T) {
 				assert.NotEmpty(t, string(message))
 
 				out := <-buildOut
+				require.Contains(t, out, "Terminal is connected, will time out in 2s...")
 				t.Log(out)
-				assert.Contains(t, out, "Terminal is connected, will time out in 2s...")
 				return
 			}
 
 			out := <-buildOut
+			require.NotContains(t, out, "Terminal is connected, will time out in 2s...")
 			t.Log(out)
-			assert.NotContains(t, out, "Terminal is connected, will time out in 2s...")
 		})
 	}
 }
@@ -1180,9 +1227,11 @@ func TestBuildWithGitCleanFlags(t *testing.T) {
 		build, cleanup := newBuild(t, jobResponse, shell)
 		defer cleanup()
 
-		build.Variables = append(build.Variables,
+		build.Variables = append(
+			build.Variables,
 			common.JobVariable{Key: "GIT_STRATEGY", Value: "fetch"},
-			common.JobVariable{Key: "GIT_CLEAN_FLAGS", Value: "-ffdx cleanup_file"})
+			common.JobVariable{Key: "GIT_CLEAN_FLAGS", Value: "-ffdx cleanup_file"},
+		)
 
 		// Run build and save file
 		err = buildtest.RunBuild(t, build)
