@@ -27,6 +27,7 @@ func (b *AbstractShell) GetFeatures(features *common.FeaturesInfo) {
 	features.RawVariables = true
 	features.ArtifactsExclude = true
 	features.MultiBuildSteps = true
+	features.VaultSecrets = true
 }
 
 func (b *AbstractShell) writeCdBuildDir(w ShellWriter, info common.ShellScriptInfo) {
@@ -105,7 +106,7 @@ func (b *AbstractShell) cacheExtractor(w ShellWriter, info common.ShellScriptInf
 			continue
 		}
 
-		b.addExtractCacheCommand(w, info, cacheFile, cacheKey)
+		b.extractCacheOrFallbackCacheWrapper(w, info, cacheFile, cacheKey)
 	}
 
 	if skipRestoreCache {
@@ -115,11 +116,26 @@ func (b *AbstractShell) cacheExtractor(w ShellWriter, info common.ShellScriptInf
 	return nil
 }
 
+func (b *AbstractShell) extractCacheOrFallbackCacheWrapper(
+	w ShellWriter,
+	info common.ShellScriptInfo,
+	cacheFile string,
+	cacheKey string,
+) {
+	cacheFallbackKey := info.Build.GetAllVariables().Get("CACHE_FALLBACK_KEY")
+
+	// Execute cache-extractor command. Failure is not fatal.
+	b.guardRunnerCommand(w, info.RunnerCommand, "Extracting cache", func() {
+		b.addExtractCacheCommand(w, info, cacheFile, cacheKey, cacheFallbackKey)
+	})
+}
+
 func (b *AbstractShell) addExtractCacheCommand(
 	w ShellWriter,
 	info common.ShellScriptInfo,
 	cacheFile string,
 	cacheKey string,
+	cacheFallbackKey string,
 ) {
 	args := []string{
 		"cache-extractor",
@@ -127,20 +143,19 @@ func (b *AbstractShell) addExtractCacheCommand(
 		"--timeout", strconv.Itoa(info.Build.GetCacheRequestTimeout()),
 	}
 
-	// Generate cache download address
 	if url := cache.GetCacheDownloadURL(info.Build, cacheKey); url != nil {
 		args = append(args, "--url", url.String())
 	}
 
-	// Execute cache-extractor command. Failure is not fatal.
-	b.guardRunnerCommand(w, info.RunnerCommand, "Extracting cache", func() {
-		w.Noticef("Checking cache for %s...", cacheKey)
-		w.IfCmdWithOutput(info.RunnerCommand, args...)
-		w.Noticef("Successfully extracted cache")
-		w.Else()
-		w.Warningf("Failed to extract cache")
-		w.EndIf()
-	})
+	w.Noticef("Checking cache for %s...", cacheKey)
+	w.IfCmdWithOutput(info.RunnerCommand, args...)
+	w.Noticef("Successfully extracted cache")
+	w.Else()
+	w.Warningf("Failed to extract cache")
+	if cacheFallbackKey != "" {
+		b.addExtractCacheCommand(w, info, cacheFile, cacheFallbackKey, "")
+	}
+	w.EndIf()
 }
 
 func (b *AbstractShell) downloadArtifacts(w ShellWriter, job common.Dependency, info common.ShellScriptInfo) {
@@ -555,6 +570,13 @@ func (b *AbstractShell) addCacheUploadCommand(
 		args = append(args, "--url", url.String())
 	}
 
+	httpHeaders := cache.GetCacheUploadHeaders(info.Build, cacheKey)
+	for key, values := range httpHeaders {
+		for _, value := range values {
+			args = append(args, "--header", fmt.Sprintf("%s: %s", key, value))
+		}
+	}
+
 	// Execute cache-archiver command. Failure is not fatal.
 	b.guardRunnerCommand(w, info.RunnerCommand, "Creating cache", func() {
 		w.Noticef("Creating cache %s...", cacheKey)
